#!/usr/bin/env python

"""
Quick BitTorrent Attack
by Javantea aka. Joel R. Voss
Feb 23, 2006

The problem with attacking BitTorrent for a banner is that you don't know what 
torrent they are seeding/downloading. If you knew that, it would be complete. 
BitTorrent is smart to not give this information out to unknown peers. The 
handshake is symmetrical, but do bittorrent peers that are connected to recv() 
first or send() first? I bet they recv() first as a security measure. If the 
person gives an incorrect info_hash, they disconnect.

This works extremely well with bthoney1.py as a full attack vector.

TODO: All the stuff after the handshake.
"""

import socket
import base64
from sha import sha
import random
from bencode import bdecode, bencode
from urllib2 import urlopen, HTTPError
from urllib import urlencode

def bittorrent_file(filename="[isoHunt] snl_lazy_sunday.mp4.torrent", tracker = False):
	
	# get the data from the torrent
	file = open(filename, 'rb')
	data = file.read()
	file.close()
	torrent_dict = bdecode(data)
	tracker_url = torrent_dict['announce']
	# announce = http://tracker.torrentbox.com:2710/announce
	print 'tracker:', tracker_url
	info_dict = torrent_dict['info']
	info_bencode = bencode(info_dict)
	info_hash = sha(info_bencode).digest()

	#print info_dict['piece length'], info_dict['name'], info_dict['private'], info_dict['length']
	# piece length = 32768, name = snl_lazy_sunday.mp4, private = 0
	#p = info_dict['pieces']
	# len(p) = 17460
	
	print 'file name, length:', info_dict['name'] + ', ' + str(info_dict['length'])
	
	if tracker == False:
		return info_hash
	else:
		return (info_hash, tracker_url)
	#end if
#end def bittorrent_file(filename='btproto1.py', tracker = True, peer_id='abcd_abcd_abcd_abcd_', ip='192.168.0.3', port='6881')

def bittorrent_tracker(tracker_url, info_hash, peer_id='abcd_abcd_abcd_abcd_', ip='192.168.0.3', port='6881'):
	# sending a request to the tracker.
	# change these values to increase/decrease your success/failure rates.
	uploaded = 0
	downloaded = 0
	left = 1234 #info_dict['length']
	event = 'empty'
	
	query = [('info_hash', info_hash), 
		('peerid', peer_id), ('ip', ip), ('port', port), 
		('uploaded', str(uploaded)), ('downloaded', str(downloaded)), 
		('left', str(left)), ('event', str(event))]
	try:
		tracker_data = urlopen(tracker_url + '?' + urlencode(query))
	except HTTPError, ex:
		print "Tracker GET failed.", ex
		return None
	#end try
	tracker_dict = bdecode(tracker_data)
	# this is an error condition, but we can still return the info_hash
	if not 'interval' in tracker_dict:
		print 'Error with tracker:'
		print tracker_data
		print tracker_dict
		return None
	#end if
	print 'interval:', tracker_dict['interval']
	
	# Why do we want to know the peers? It's interesting.
	peers = tracker_dict['peers']
	for peer in peers:
		print peer['ip'] + ':' + peer['port'], peer['peer_id']
	#next peer
	
	return peers
#end def bittorrent_tracker(tracker_url, info_hash, peer_id='abcd_abcd_abcd_abcd_', ip='192.168.0.3', port='6881')

def bittorrent_client(host='127.0.0.1', port=6881, filename="[isoHunt] snl_lazy_sunday.mp4.torrent"):
	""" 
	This simple script connects to a bittorrent peer and attempts to handshake 
	The BitTorrent protocol is very easy to implement.
	"""
	
	# generate 20 byte random peer_id
	peer_id = ""
	for i in range(20): peer_id += chr(random.randint(0, 255))
	
	# info hash
	# d = base64.b16decode("7DCF2A0FF459A6E731372B39C0E2358539CE2A12")
	#file = open(filename, "rb")
	#data = file.read(256 * 1024)
	#file.close()
	#info_hash = sha(data).digest()
	#data = ""
	# intentionally use the attackee's ip and port to confuse the tracker.
	info_hash = bittorrent_file(filename)
	# if you want to see the tracker data, do this:
	#(info_hash, tracker_url) = bittorrent_file(filename, True)
	#peers = bittorrent_tracker(tracker_url, info_hash, peer_id, host, port)
	
	# connect to the server
	s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	s.connect((host, port))
	bittorrent_handshake(s, info_hash, peer_id)
	s.close()
#end def bittorrent_client()

def bittorrent_handshake(s, info_hash, peer_id):
	# . is a length parameter, "BitTorrent protocol" is a header.
	# supposedly, I could do chr(49) + "This is a very long protocol name"
	bt_header = ".BitTorrent protocol"
	# 8 bytes of nulls reserved
	reserved = "\x00\x00\x00\x00\x00\x00\x00\x00"
	
	# handshake is symmetric no matter which peer you are.
	s.send(bt_header+reserved+info_hash+peer_id)
	hs_len = len(bt_header+reserved+info_hash+peer_id)
	print "Sent", hs_len, "B"
	output = s.recv(hs_len)
	print urlencode([('output', output)])
#end def bittorrent_handshake(s, info_hash, peer_id)

def main():
	bittorrent_client()
#end def main()

if __name__ == '__main__':
	main()
#end if
